@@ -59,6 +59,11 @@ int main(int argc, const char *argv[])
 
     err = ipkg_cmd_exec(cmd, &ipkg_conf, argc - optind, argv + optind);
 
+    if ( err == 0 ) {
+       ipkg_message(ipkg_conf, IPKG_NOTICE, "Succesfully done.\n");
+    } else {
+       ipkg_message(ipkg_conf, IPKG_NOTICE, "Error returned. Return value is %d\n.",err);
+    }
     /* XXX: FEATURE request: run ldconfig and/or depmod after package needing them are installed or removed */  
     // ipkg_global_postinst();
 
