@@ -26,10 +26,12 @@
 #include "ipkg_message.h"
 #include "file_util.h"
 #include "str_util.h"
+#include "xsystem.h"
 
 static int ipkg_conf_parse_file(ipkg_conf_t *conf, const char *filename,
 				pkg_src_list_t *pkg_src_list,
-				nv_pair_list_t *tmp_dest_nv_pair_list);
+				nv_pair_list_t *tmp_dest_nv_pair_list,
+				char *tmp_lists_dir);
 static int ipkg_init_options_array(const ipkg_conf_t *conf, ipkg_option_t **options);
 static int ipkg_conf_set_option(const ipkg_option_t *options,
 				const char *name, const char *value);
@@ -38,7 +40,7 @@ static int ipkg_conf_set_default_dest(ipkg_conf_t *conf,
 static int set_and_load_pkg_src_list(ipkg_conf_t *conf,
 				     pkg_src_list_t *nv_pair_list);
 static int set_and_load_pkg_dest_list(ipkg_conf_t *conf,
-				      nv_pair_list_t *nv_pair_list);
+				      nv_pair_list_t *nv_pair_list, char * lists_dir);
 
 int ipkg_init_options_array(const ipkg_conf_t *conf, ipkg_option_t **options)
 {
@@ -62,7 +64,6 @@ int ipkg_init_options_array(const ipkg_conf_t *conf, ipkg_option_t **options)
 	  { "proxy_user", IPKG_OPT_TYPE_STRING, &conf->proxy_user },
 	  { "query-all", IPKG_OPT_TYPE_BOOL, &conf->query_all },
 	  { "verbose-wget", IPKG_OPT_TYPE_BOOL, &conf->verbose_wget },
-	  { "verbose_wget", IPKG_OPT_TYPE_BOOL, &conf->verbose_wget },
 	  { "verbosity", IPKG_OPT_TYPE_BOOL, &conf->verbosity },
 	  { NULL }
      };
@@ -100,8 +101,10 @@ int ipkg_conf_init(ipkg_conf_t *conf, const args_t *args)
      int err;
      char *tmp_dir_base;
      nv_pair_list_t tmp_dest_nv_pair_list;
+     char * lists_dir =NULL;
      glob_t globbuf;
      char *etc_ipkg_conf_pattern = "/etc/ipkg/*.conf";
+     char *pending_dir  =NULL;
 
      memset(conf, 0, sizeof(ipkg_conf_t));
 
@@ -115,8 +118,6 @@ int ipkg_conf_init(ipkg_conf_t *conf, const args_t *args)
      conf->restrict_to_default_dest = 0;
      conf->default_dest = NULL;
 
-     conf->lists_dir = IPKG_CONF_LISTS_DIR;
-     conf->pending_dir = IPKG_CONF_PENDING_DIR;
 
      if (args->tmp_dir)
 	  tmp_dir_base = args->tmp_dir;
@@ -157,18 +158,31 @@ int ipkg_conf_init(ipkg_conf_t *conf, const args_t *args)
      pkg_hash_init("pkg-hash", &conf->pkg_hash, IPKG_CONF_DEFAULT_HASH_LEN);
      hash_table_init("file-hash", &conf->file_hash, IPKG_CONF_DEFAULT_HASH_LEN);
      hash_table_init("obs-file-hash", &conf->obs_file_hash, IPKG_CONF_DEFAULT_HASH_LEN);
+     lists_dir=(char *)malloc(1);
      if (args->conf_file) {
 	  struct stat stat_buf;
 	  err = stat(args->conf_file, &stat_buf);
 	  if (err == 0)
 	       if (ipkg_conf_parse_file(conf, args->conf_file,
-				    &conf->pkg_src_list, &tmp_dest_nv_pair_list)<0) {
+				    &conf->pkg_src_list, &tmp_dest_nv_pair_list,lists_dir)<0) {
                    /* Memory leakage from ipkg_conf_parse-file */
                    return -1;
                }
                    
      }
 
+     /* if (!lists_dir ){*/
+     if (strlen(lists_dir)<=1 ){
+        lists_dir = realloc(lists_dir,strlen(IPKG_CONF_LISTS_DIR));
+        sprintf (lists_dir,"%s",IPKG_CONF_LISTS_DIR);
+     }
+
+     pending_dir = malloc(strlen(lists_dir)+strlen("/pending")+2);
+     snprintf(pending_dir,strlen(lists_dir)+strlen("/pending") ,"%s%s",lists_dir,"/pending");
+
+     conf->lists_dir = strdup(lists_dir);
+     conf->pending_dir = strdup(pending_dir);
+
      if (args->offline_root) 
 	  sprintf_alloc(&etc_ipkg_conf_pattern, "%s/etc/ipkg/*.conf", args->offline_root);
      memset(&globbuf, 0, sizeof(globbuf));
@@ -178,7 +192,7 @@ int ipkg_conf_init(ipkg_conf_t *conf, const args_t *args)
 	  for (i = 0; i < globbuf.gl_pathc; i++) {
 	       if (globbuf.gl_pathv[i]) 
 		    if ( ipkg_conf_parse_file(conf, globbuf.gl_pathv[i], 
-					 &conf->pkg_src_list, &tmp_dest_nv_pair_list)<0 ){
+				         &conf->pkg_src_list, &tmp_dest_nv_pair_list,lists_dir)<0) {
                         /* Memory leakage from ipkg_conf_parse-file */
                         return -1;
 	            }
@@ -261,7 +275,7 @@ int ipkg_conf_init(ipkg_conf_t *conf, const args_t *args)
         /* Now that we have resolved conf->offline_root, we can commit to
 	   the directory names for the dests and load in all the package
 	   lists. */
-        set_and_load_pkg_dest_list(conf, &tmp_dest_nv_pair_list);
+        set_and_load_pkg_dest_list(conf, &tmp_dest_nv_pair_list,lists_dir);
    
         if (args->dest) {
 	     err = ipkg_conf_set_default_dest(conf, args->dest);
@@ -271,6 +285,8 @@ int ipkg_conf_init(ipkg_conf_t *conf, const args_t *args)
         }
      }
      nv_pair_list_deinit(&tmp_dest_nv_pair_list);
+     free(lists_dir);
+     free(pending_dir);
 
      return 0;
 }
@@ -298,8 +314,17 @@ void ipkg_conf_deinit(ipkg_conf_t *conf)
      }
 #endif /* IPKG_DEBUG_NO_TMP_CLEANUP */
 
+     free(conf->tmp_dir); /*XXX*/
+
      pkg_src_list_deinit(&conf->pkg_src_list);
      pkg_dest_list_deinit(&conf->pkg_dest_list);
+     nv_pair_list_deinit(&conf->arch_list);
+     if (&conf->pkg_hash)
+	            pkg_hash_deinit(&conf->pkg_hash);
+     if (&conf->file_hash)
+	            hash_table_deinit(&conf->file_hash);
+     if (&conf->obs_file_hash)
+	            hash_table_deinit(&conf->obs_file_hash);
 
      ipkg_conf_free_string(&conf->offline_root);
      ipkg_conf_free_string(&conf->offline_root_pre_script_cmd);
@@ -386,7 +411,7 @@ static int set_and_load_pkg_src_list(ipkg_conf_t *conf, pkg_src_list_t *pkg_src_
      return 0;
 }
 
-static int set_and_load_pkg_dest_list(ipkg_conf_t *conf, nv_pair_list_t *nv_pair_list)
+static int set_and_load_pkg_dest_list(ipkg_conf_t *conf, nv_pair_list_t *nv_pair_list, char *lists_dir )
 {
      nv_pair_list_elt_t *iter;
      nv_pair_t *nv_pair;
@@ -401,7 +426,7 @@ static int set_and_load_pkg_dest_list(ipkg_conf_t *conf, nv_pair_list_t *nv_pair
 	  } else {
 	       root_dir = strdup(nv_pair->value);
 	  }
-	  dest = pkg_dest_list_append(&conf->pkg_dest_list, nv_pair->name, root_dir);
+	  dest = pkg_dest_list_append(&conf->pkg_dest_list, nv_pair->name, root_dir, lists_dir);
 	  free(root_dir);
 	  if (dest == NULL) {
 	       continue;
@@ -420,7 +445,8 @@ static int set_and_load_pkg_dest_list(ipkg_conf_t *conf, nv_pair_list_t *nv_pair
 
 static int ipkg_conf_parse_file(ipkg_conf_t *conf, const char *filename,
 				pkg_src_list_t *pkg_src_list,
-				nv_pair_list_t *tmp_dest_nv_pair_list)
+				nv_pair_list_t *tmp_dest_nv_pair_list,
+				char *lists_dir)
 {
      int err;
      ipkg_option_t * options;
@@ -435,6 +461,7 @@ static int ipkg_conf_parse_file(ipkg_conf_t *conf, const char *filename,
      if (file == NULL) {
 	  fprintf(stderr, "%s: failed to open %s: %s\n",
 		  __FUNCTION__, filename, strerror(errno));
+	  free(options);
 	  return errno;
      }
      ipkg_message(conf, IPKG_NOTICE, "loading conf file %s\n", filename);
@@ -443,10 +470,12 @@ static int ipkg_conf_parse_file(ipkg_conf_t *conf, const char *filename,
 		    "^[[:space:]]*(#.*|[[:space:]]*)$",
 		    REG_EXTENDED);
      if (err) {
+	  free(options);
 	  return err;
      }
      err = xregcomp(&valid_line_re, "^[[:space:]]*(\"([^\"]*)\"|([^[:space:]]*))[[:space:]]*(\"([^\"]*)\"|([^[:space:]]*))[[:space:]]*(\"([^\"]*)\"|([^[:space:]]*))([[:space:]]+([^[:space:]]+))?[[:space:]]*$", REG_EXTENDED);
      if (err) {
+	  free(options);
 	  return err;
      }
 
@@ -526,6 +555,14 @@ static int ipkg_conf_parse_file(ipkg_conf_t *conf, const char *filename,
 	       }
 	  } else if (strcmp(type, "dest") == 0) {
 	       nv_pair_list_append(tmp_dest_nv_pair_list, name, value);
+	  } else if (strcmp(type, "lists_dir") == 0) {
+	       lists_dir = realloc(lists_dir,strlen(value)+1);
+               sprintf (lists_dir,"%s",value);
+               if (lists_dir == NULL) {
+		    ipkg_message(conf, IPKG_ERROR, "ERROR: Not enough memory\n");
+	            free(options);
+	            return EINVAL;
+               }
 	  } else if (strcmp(type, "arch") == 0) {
 	       ipkg_message(conf, IPKG_INFO, "supported arch %s priority (%s)\n", name, value);
 	       if (!value) {
@@ -536,6 +573,7 @@ static int ipkg_conf_parse_file(ipkg_conf_t *conf, const char *filename,
 	  } else {
 	       fprintf(stderr, "WARNING: Ignoring unknown configuration "
 		       "parameter: %s %s %s\n", type, name, value);
+	       free(options);
 	       return EINVAL;
 	  }
 
@@ -549,6 +587,7 @@ static int ipkg_conf_parse_file(ipkg_conf_t *conf, const char *filename,
 	  free(line);
      }
 
+     free(options);
      regfree(&comment_re);
      regfree(&valid_line_re);
      fclose(file);
